-- This code by StewartML
-- finished on 04/jan/04
-- contact: stewart@isoclass.co.uk

include graphics.e
include image.e
include get.e

sequence SinTable, CosTable, Screen, Map
integer PlayerX, PlayerY, PlayerA

constant ScreenWidth  = 320,
         ScreenHeight = 200,
         FeildOfVeiw  = 60,
         GraphicsMode = 13,

         GridSize = 11,
         Magic    = 1500,

         HalfScreenWidth  = ScreenWidth  / 2,
         HalfScreenHeight = ScreenHeight / 2,
         HalfFeildOfVeiw  = FeildOfVeiw  / 2,

         FeildsOfVeiwPerRotation = 360 / FeildOfVeiw,
         NumberOfScreenDegrees   = ScreenWidth * FeildsOfVeiwPerRotation,

         NumberOfDegreesInScreenDegree = 360 / NumberOfScreenDegrees,
         NumberOfScreenDegreesInDegree = NumberOfScreenDegrees / 360



function Rad(atom deg)
    return deg * (PI / 180)
end function

function Scr(atom deg)
    return floor(deg * NumberOfScreenDegreesInDegree)
end function

function ScrToDeg(integer scr)
    return floor(scr * NumberOfDegreesInScreenDegree)
end function

function Round(atom n)
    integer a, b
    a = floor(n)
    b = floor(n+1)
    if (b-n) < (n-a) then
        return b
    else
        return a
    end if
end function

function FixAngle(integer angle)
    if angle < 1 then
        return angle + NumberOfScreenDegrees

    elsif angle > NumberOfScreenDegrees then

        return angle - NumberOfScreenDegrees
    else
        return angle
    end if
end function


procedure MakeTables()
    integer fn

    SinTable = repeat(0, NumberOfScreenDegrees)
    CosTable = SinTable    

    for i = 1 to NumberOfScreenDegrees do
        SinTable[i] = sin(Rad(i * NumberOfDegreesInScreenDegree))
        CosTable[i] = cos(Rad(i * NumberOfDegreesInScreenDegree))
    end for

    fn = open("mapdata","r")
    Map = get(fn)
    if fn = -1 or (not Map[1]) then
        puts(1,"Invalid or unfound map data.\n")
    end if
    Map = Map[2]
    close(fn)    
end procedure


procedure ClearScreen()
    Screen = repeat(repeat(0,ScreenWidth),ScreenHeight)
end procedure

procedure DrawColumn(integer c, integer sx, integer sy, integer len, integer w)
    if sx < 1 then sx = 1 elsif sx > ScreenWidth then sx = ScreenWidth end if
    if sy < 1 then sy = 1 elsif sy > ScreenHeight then sy = ScreenHeight end if
    if sy + len > ScreenHeight then len = ScreenHeight - sy end if

    for i = 1 to len do
        Screen[sy+i][sx..sx+w-1] = c
    end for
end procedure


function CastRay(integer px, integer py, integer pa, integer ra)
    integer Angle, CellData, Distance
    atom rx, ry

    Angle = FixAngle(pa + ra)

    rx = px
    ry = py

    CellData = 0
    Distance = 0

    while CellData = 0 do
        rx = rx - SinTable[Angle] 
        ry = ry - CosTable[Angle] 

        CellData = Map[Round(ry / GridSize)][Round(rx / GridSize)]

        Distance = Distance + 1       
    end while

    return {Distance, CellData}
end function


procedure RenderVeiw(integer px, integer py, integer pa)
    sequence Ray
    integer Height

    ClearScreen()   

    for i = -HalfScreenWidth to HalfScreenWidth - 1 do
        Ray = CastRay(px, py, pa, i)
        Height = Round(Magic / Ray[1])                
        DrawColumn(Ray[2], i + HalfScreenWidth,
            floor(HalfScreenHeight - Height / 2), Height, 1)
    end for

    display_image({0,0}, Screen)
end procedure


procedure Init()
    object x
    x = graphics_mode(GraphicsMode)

    PlayerX = 3 * GridSize
    PlayerY = 3 * GridSize
    PlayerA = Scr(0)
end procedure

procedure Pause()
    while get_key() = -1 do end while
end procedure

procedure Exit()
    object x
    x = graphics_mode(-1)
    abort(0)
end procedure


integer Render, k, nx, ny
Render = 1

MakeTables()
ClearScreen()
Init()

while 1 do
    if Render then RenderVeiw(PlayerX, PlayerY, PlayerA) end if
    k = get_key()

    Render = 1

    if k = 331 then
        PlayerA = FixAngle(PlayerA - Scr(5))

    elsif k = 333 then
        PlayerA = FixAngle(PlayerA + Scr(5))

    elsif k = 328 then
        nx = Round(PlayerX - SinTable[PlayerA] * 2)
        ny = Round(PlayerY - CosTable[PlayerA] * 2)
        if Map[floor(ny/10)][floor(nx/10)] = 0 then
            PlayerX = nx
            PlayerY = ny
        end if

    elsif k = 27  then Exit()
    else Render = 0 end if
end while

Pause()
Exit()
